uses com

constant
  dbOpenTable = $00000001
  dbOpenDynaset = $00000002
  dbOpenSnapshot = $00000004
  dbOpenForwardOnly = $00000008
  dbOpenDynamic = $00000010
end

constant
  dbBoolean = $00000001
  dbByte = $00000002
  dbInteger = $00000003
  dbLong = $00000004
  dbCurrency = $00000005
  dbSingle = $00000006
  dbDouble = $00000007
  dbDate = $00000008
  dbBinary = $00000009
  dbText = $0000000A
  dbLongBinary = $0000000B
  dbMemo = $0000000C
  dbGUID = $0000000F
  dbBigInt = $00000010
  dbVarBinary = $00000011
  dbChar = $00000012
  dbNumeric = $00000013
  dbDecimal = $00000014
  dbFloat = $00000015
  dbTime = $00000016
  dbTimeStamp = $00000017
end

function Debug() external

function DAO_OpenDatabase( DatabaseName )
   object DBEngine, Workspaces, OpenDatabase : Dispatch
   DBEngine.Init( 'DAO.DBEngine.36' )
return DBEngine/Workspaces[0]/OpenDatabase[ DatabaseName ]

function DAO_GetFieldRef( RS, FieldName )
  object Fields : Dispatch
return RS/Fields[ FieldName ]

function DAO_ReadField( RS, FieldName )
return GetProperty( DAO_GetFieldRef( RS, FieldName ), 'Value', [] )

class Recordset
  function Open( DB, RecordsetName, Mode ) constructor
  function Close()
  function "[]"( N )
  function PutField( FieldName, Value )
  function GetField( FieldName )
  function AddNew()
  function Edit()
  function Delete()
  function Update()
  function CancelUpdate()
  function SetIndex( IndexName )
  function GetIndex()
  function Seek( Operator, Value )
  function NoMatch()
  function Move( N )
  function MoveFirst()
  function MoveLast()
  function MoveNext()
  function MovePrevious()
  function FindFirst( Expression )
  function FindLast( Expression )
  function FindNext( Expression )
  function FindPrevious( Expression )
  function EOF()
  function BOF()
  function Bookmark()
  function MoveBookmark( B )
  function RecordCount()
  function Transactions()
  function Type()
  function Updatable()
  function Bookmarkable()
end

implementation Recordset
  local RS

  function Open( DB, RecordsetName, Mode )
    RS := MethodCall( DB, 'OpenRecordset', [ RecordsetName, Mode ] )
  return RS 

  function Close()
  return MethodCall( RS, 'Close', [] )

  function "[]"( N )
  return DAO_GetFieldRef( RS, N )

  function PutField( FieldName, Value )
    PutProperty( DAO_GetFieldRef( RS, FieldName ), 'Value', [Value] )
  return 0

  function GetField( FieldName )
  return GetProperty( DAO_GetFieldRef( RS, FieldName ), 'Value', [] ) 

  function AddNew()
  return MethodCall( RS, 'AddNew', [] )

  function Edit()
  return MethodCall( RS, 'Edit', [] )

  function Delete()
  return MethodCall( RS, 'Delete', [] )

  function Update()
  return MethodCall( RS, 'Update', [] )

  function CancelUpdate()
  return MethodCall( RS, 'CancelUpdate', [] )

  function SetIndex( IndexName )
  return PutProperty( RS, 'Index', [ IndexName ] )

  function GetIndex()
  return GetProperty( RS, 'Index', [] )

  function Seek( Operator, Value )
  return MethodCall( RS, 'Seek', [Operator, Value] )

  function NoMatch()
  return GetProperty( RS, 'NoMatch', [] )

  function Transactions()
  return GetProperty( RS, 'Transactions', [] )

  function Type()
  return GetProperty( RS, 'Type', [] )

  function Move( N )
  return MethodCall( RS, 'Move', [N] )

  function FindFirst( Expression )
  return MethodCall( RS, 'FindFirst', [ Expression ] )

  function FindLast( Expression )
  return MethodCall( RS, 'FindLast', [ Expression ] )

  function FindNext( Expression )
  return MethodCall( RS, 'FindNext', [ Expression ] )

  function FindPrevious( Expression )
  return MethodCall( RS, 'FindPrevious', [ Expression ] )

  function MoveFirst()
  return MethodCall( RS, 'MoveFirst', [] )

  function MoveLast()
  return MethodCall( RS, 'MoveLast', [] )

  function MoveNext()
  return MethodCall( RS, 'MoveNext', [] )

  function MovePrevious()
  return MethodCall( RS, 'MovePrevious', [] )

  function EOF()
  return GetProperty( RS, 'EOF', [] )

  function BOF()
  return GetProperty( RS, 'BOF', [] )
 
  function Bookmark()
  return GetProperty( RS, 'Bookmark', [] )

  function MoveBookmark( B )
  return PutProperty( RS, 'Bookmark', [B] )

  function RecordCount()
  return GetProperty( RS, 'RecordCount', [] )

  function Updatable()
  return GetProperty( RS, 'Updatable', [] )

  function Bookmarkable()
  return GetProperty( RS, 'Bookmarkable', [] )
end

--------------- Main program ---------------------

CoInitialize()

try
  DB := DAO_OpenDatabase( 'db1.mdb' )

  object RS( DB, 'select * from ATable', dbOpenSnapshot ) : Recordset

  writeln( RS.RecordCount() )
  writeln( RS.Transactions() )
  writeln( RS.Type() )
  writeln( RS.Updatable() )
  writeln( RS.Bookmarkable() )

  S := RS.GetField( 'AField' )
  writeln( S )

  RS.FindFirst( 'AField = "uuu" ' )

  if RS.NoMatch() then
    writeln( 'The record is not found' )
  else
    writeln( 'The record is found' )
  end

  RS.Close()

  Destroy( RS )

  writeln( 'Press any key' )
  Readln()

  object RS( DB, 'ATable', dbOpenTable ) : Recordset

  RS.SetIndex( 'PrimaryKey' )
  RS.Seek( '=', 2 )

  if RS.NoMatch() then
    writeln( 'The record is not found' )
  else
    writeln( 'The record is found' )
  end

  RS.AddNew()
  RS.PutField( 'AField', 'uuu' )
  RS.Update()
  
  I := 0
  RS.MoveFirst()
  while not RS.EOF() do
    Inc(I)
    writeln( 'Record number ' + STR(I) )
    writeln( RS.GetField(0) ) 
    writeln( RS.GetField(1) ) 
    RS.MoveNext()
  end

  RS.MovePrevious()
  RS.Edit()
  RS.PutField( 'AField', 'ppp' )
  RS.Update()

  RS.MoveLast()
  RS.Delete()

  RS.Move( -2 )
  writeln( RS.GetField(0) ) 

  B := RS.Bookmark()

  RS.MoveFirst()
  writeln( RS.GetField(0) ) 
  RS.MoveBookmark( B )

  writeln( RS.GetField(0) ) 

finally

  CoUninitialize()

  Debug()

  writeln( 'COM closed' )
end

return
